/*
Build by Wiquid's PCI Generator for TAO platform Free to use
 */
define([
	'qtiCustomInteractionContext',
	'jquery',
	'./js/renderer',
	'OAT/util/event',
	'i18n'],
	function (qtiCustomInteractionContext, $, renderer, event, __) {
		'use strict';

		/**
		 * オブジェクトかどうかをチェック
		 * 
		 * @param {*} value 
		 * @returns Boolean
		 */
		var isObject = function (value) {
			return ((value !== null) && (value !== undefined) && (Object.prototype.toString.call(value) === '[object Object]')) ? true : false;
		}

		/**
		 * JSONかどうかをチェック
		 * 
		 * @param {*} data 
		 * @returns Boolean
		 */
		var isJson = function (data) {
			try {
				JSON.parse(data);
			} catch (error) {
				return false;
			}
			return true;
		}

		var PivotTableInteraction = {
			id: -1,
			getTypeIdentifier: function getTypeIdentifier() {
				return 'PivotTableInteraction';
			},
			/**
			 * Render the PCI :
			 * @param {String} id
			 * @param {Node} dom
			 * @param {Object} config - json
			 */
			initialize: function initialize(id, dom, config, assetManager) {
				//add method on(), off() and trigger() to the current object
				event.addEventMgr(this);
				var _this = this;
				this.id = id;
				this.dom = dom;
				this.config = config || {};
				renderer.render(this.id, this.dom, this.config, assetManager);
				//tell the rendering engine that I am ready
				qtiCustomInteractionContext.notifyReady(this);
				//listening to dynamic configuration change
				this.on('csvdatachange', function (csvData) {
					_this.config.csvData = csvData;
				});
			},
			/**
			 * Programmatically set the response following the json schema described in
			 * http://www.imsglobal.org/assessment/pciv1p0cf/imsPCIv1p0cf.html#_Toc353965343
			 *
			 * @param {Object} interaction
			 * @param {Object} response
			 */
			setResponse: function setResponse(response) {
				var Scontainer = $(this.dom);
				// パラメータがJSONだったらそのまま、オブジェクトだったらJSONに変換、どちらでもなかったら空のオブジェクトをJSONに変換して使用
				response = (isJson(response)) ? response : ((isObject(response)) ? JSON.stringify(response) : JSON.stringify({}));
				// 操作ログのhiddenフォームに格納
				Scontainer.find('input.operation-log').val(response);
				console.log(this.getTypeIdentifier() + " PivotTable.amd.js setResponse:" + JSON.stringify(response));
			},
			/**
			 * Get the response in the json format described in
			 * http://www.imsglobal.org/assessment/pciv1p0cf/imsPCIv1p0cf.html#_Toc353965343
			 * ※この値が回答としてPOSTされます。
			 *
			 * @param {Object} interaction
			 * @returns {Object}
			 */
			getResponse: function getResponse() {
				var Scontainer = $(this.dom);
				/* buildHighlightIndexが多数実行されることに対する処置 */
				Scontainer.addClass('select2-container');
				/* 回答として操作ログを返す(操作ログを取得しない設定の場合は空の配列を返す) */
				var value = Scontainer.find('input.operation-log').val();
				console.log(this.getTypeIdentifier() + " PivotTable.amd.js getResponse:" + JSON.stringify({ base: { string: value } }));
				return { base: { string: value } };
			},
			/**
			 * Remove the current response set in the interaction
			 * The state may not be restored at this point.
			 *
			 * @param {Object} interaction
			 */
			resetResponse: function resetResponse() {
				var Scontainer = $(this.dom);
				Scontainer.find('input.operation-log').val('[]');
				console.log(this.getTypeIdentifier() + " PivotTable.amd.js resetResponse");
			},
			/**
			 * Reverse operation performed by render()
			 * After this function is executed, only the inital naked markup remains
			 * Event listeners are removed and the state and the response are reset
			 *
			 * @param {Object} interaction
			 */
			destroy: function destroy() {
				var Scontainer = $(this.dom);
				Scontainer.off().empty();
			},
			/**
			 * Restore the state of the interaction from the serializedState.
			 *
			 * @param {Object} interaction
			 * @param {Object} serializedState - json format
			 */
			setSerializedState: function setSerializedState(state) {
				// パラメータがJSONでなければJSONに変換
				state = (isJson(state)) ? state : ((isObject(state)) ? JSON.stringify(state) : JSON.stringify({}));
				console.log(this.getTypeIdentifier() + " PivotTable.amd.js setSerializedState:" + state);
				// JSON形式の設定をオブジェクトに変換
				var pivotTableStates = JSON.parse(state);
				// 回答情報が渡されていないかどうか確認(TAOの結果画面→[レビュー]で発生する様子)
				if (('response' in pivotTableStates) && ('base' in pivotTableStates.response) && ('string' in pivotTableStates.response.base)) {
					console.warn(this.getTypeIdentifier() + " PivotTable.amd.js setSerializedState:The response was passed.");
					// 回答情報が渡されていた場合はsetResponseを実行
					this.setResponse(pivotTableStates.response.base.string);
					return;
				}
				// クロス集計表の設定を復元
				renderer.setPivotTableStates(this.id, this.dom, this.config, pivotTableStates);
				return;
			},
			/**
			 * Get the current state of the interaction as a string.
			 * It enables saving the state for later usage.
			 *
			 * @param {Object} interaction
			 * @returns {Object} json format
			 */
			getSerializedState: function getSerializedState() {
				// 現在の集計表の設定を取得
				var pivotTableStates = renderer.getPivotTableStates(this.id, this.dom, this.config);
				// JSON形式に変換
				var pivotTableStatesJSON = JSON.stringify(pivotTableStates);
				console.log(this.getTypeIdentifier() + " PivotTable.amd.js getSerializedState:" + pivotTableStatesJSON);
				// JSON形式で応答を返す
				return pivotTableStatesJSON;
			}
		};
		qtiCustomInteractionContext.register(PivotTableInteraction);
	}
);
